/*
* main.c - contains program main
*
* Copyright (c) 2014-2019 emotas embedded communication GmbH
*-------------------------------------------------------------------
* $Id: main.c 33490 2020-09-28 14:13:46Z hil $
*
*
*-------------------------------------------------------------------
*
*
*/

/********************************************************************/
/**
* \file
* \brief main routine
*
*/

/* header of standard C - libraries
---------------------------------------------------------------------------*/
#include <stdio.h>

/* header of project specific types
---------------------------------------------------------------------------*/
#include <gen_define.h>

#include <j1939_j1939.h>
#include <gen_extern_decl.h>

/* constant definitions
---------------------------------------------------------------------------*/

/* local defined data types
---------------------------------------------------------------------------*/

/* list of external used functions, if not in headers
--------------------------------------------------------------------------*/

/* list of global defined functions
---------------------------------------------------------------------------*/

/* list of local defined functions
---------------------------------------------------------------------------*/
static void errorHandler(UNSIGNED8 error);
static UNSIGNED8 claimAddress(BOOL_T valid, UNSIGNED8 ownClaimAddr);
static RET_T transmitPgn(UNSIGNED32  pgn);
static void commInd(CO_COMM_STATE_EVENT_T commEvent);
static void canInd(CO_CAN_STATE_T canState);
static void receivePgn(UNSIGNED32 pgn, UNSIGNED8 node, J1939_RECEIVE_STATE_T state);

/* external variables
---------------------------------------------------------------------------*/

/* global variables
---------------------------------------------------------------------------*/
/* PGN 53248 CL Cab Illumination Message */
UNSIGNED8	ilbp;	/* Illumination Brightness Percent */
UNSIGNED8	sbbp;	/* Switch Backlight Illumination Brightness Percent */
UNSIGNED8	sibp;	/* Switch Indication Illumination Brightness Percent */

/* PGN 64956 TR4 Farebox Service Detail */
UNSIGNED8	FareboxServiceStatus = 1;	/* Farebox Service Status */
UNSIGNED8	TripStatus = 5;	/* Trip Status */
UNSIGNED8	dummy3;
UNSIGNED8	TripDirection = 9;	/* Trip Direction */
UNSIGNED8	dummy4;
UNSIGNED8	FarePresets = 0x7f;	/* Fare Presets */
UNSIGNED16	tripNumber = 0x1234;	/* Trip Number */
UNSIGNED16	PatternNumber = 0x5678;	/* Pattern Number */
UNSIGNED16	Assignedroute = 0x90ab;	/* Assigned Route */
UNSIGNED16	AssignedRun = 0x55aa;	/* Assigned Run */
UNSIGNED16	AssignedBlock = 0xcdef;	/* Assigned Block */
UNSIGNED16	DriversFareboxSecurityCode = 0x8037;

/* PGN 65128 VF Vehicle Fluids */
UNSIGNED8	HydraulicTemperature;	/* Hydraulic Temperature */
UNSIGNED8	HydraulicOilFilterRestrictionSwitch;	/* Hydraulic Oil Filter Restriction Switch */
UNSIGNED8	WinchOilPressureSwitch;	/* Winch Oil Pressure Switch */
UNSIGNED8	HydraulicOilLevel;	/* Hydraulic Oil Level */

/* PGN 65212 CBI Compression/Service Brake Information */
UNSIGNED32	TotalCompressionBrakeDistance;	/* Total Compression Brake Distance */
UNSIGNED32	TripCompressionBrakeDistance;	/* Trip Compression Brake Distance */
UNSIGNED32	TripSeviceBrakeApplication;	/* Trip Service Brake Distance */
UNSIGNED32	TripSeviceBrakeDistance;	/* Trip Service Brake Applications */

/* PGN 65262 ET1 Engine Temperature 1 */
UNSIGNED8	ect;	/* Engine Coolant Temperature */
UNSIGNED8	ft;	/* Engine Fuel 1 Temperature 1 */
UNSIGNED16	eol;	/* Engine Oil Temperature 1 */
UNSIGNED16	tol;	/* Engine Turbocharger Oil Temperature */
UNSIGNED8	eit;	/* Engine Intercooler Temperature */
UNSIGNED8	eito;	/* Engine Charge Air Cooler Thermostat Opening */

UNSIGNED16	eta;

UNSIGNED8	ef1t1;
UNSIGNED16	eot1;
UNSIGNED16	etot;
UNSIGNED8	ecacto;

/* local defined variables
---------------------------------------------------------------------------*/


/***************************************************************************/
/**
* \brief main entry
*
* \param
*	nothing
* \results
*	nothing
*/
int main(
		void
	)
{
	/* hardware initialization */
	j1939drvHardwareInit();

	if (j1939drvCanInit(250u) != RET_OK)  {
		errorHandler(1);
	}

	if (j1939drvTimerSetup(J1939_TIMER_INTERVAL) != RET_OK)  {
		errorHandler(2);
	}

	/* stack init */
	if (j1939StackInit() != RET_OK)  {
		errorHandler(3);
	}

	/* enable CAN communication */
	if (j1939drvCanEnable() != RET_OK)  {
		errorHandler(4);
	}

	/* setup event registration */
	if (j1939EventRegister_COMM_EVENT(commInd) != RET_OK)  {
		errorHandler(5);
	}

	if (j1939EventRegister_CAN_STATE(canInd) != RET_OK)  {
		errorHandler(6);
	}

	if (j1939EventRegister_CLAIM_ADDRESS(claimAddress) != RET_OK)  {
		errorHandler(7);
	}

	if (j1939EventRegister_TRANSMIT_PGN(transmitPgn) != RET_OK)  {
		errorHandler(8);
	}

	if (j1939EventRegister_RECEIVE_PGN(receivePgn) != RET_OK)  {
		errorHandler(9);
	}


	if (j1939ChangeDestinationAddress(53248u, 22u) != RET_OK)  {
		errorHandler(10);
	}


	while (1)  {
		
		while (j1939CommTask() == CO_TRUE)  {
			/* more messages to process
			 * call j1939CommTask() again */
		}

		/* appl(); */
	}
}

/*********************************************************************/
/*
 * registered function for claim address
 */
static UNSIGNED8 claimAddress(
		BOOL_T valid,				/* claim address valid */
		UNSIGNED8	claimAddr		/* claim address */
	)
{
UNSIGNED8 newAddress = claimAddr;

	if (valid == CO_TRUE)  {
		printf("Claim Address %d is valid now...\n", claimAddr);
	} else {
		printf("claim indication old address: %d invalid, ", claimAddr);
		if (claimAddr < 220u)  {
			newAddress = claimAddr + 1u;
		} else {
			return(255u);
		}
		printf("try new claim Address %d\n", newAddress);
	}

	return(newAddress);
}


/*********************************************************************/
/*
 * registered function for receive PGNs
 * (Called if PGN was received)
 */
static void receivePgn(
		UNSIGNED32  pgn,
		UNSIGNED8	node,
		J1939_RECEIVE_STATE_T state
	)
{
	printf("PGN %d received from node %d - state %d\n", pgn, node, state);

	if (pgn == 65128u)  {
		printf("HydraulicTemperature: %d °C\n",
			j1939Slot67_toDisp(HydraulicTemperature));
		printf("HydraulicOilFilterRestrictionSwitch: %d\n",
			HydraulicOilFilterRestrictionSwitch);
		printf("WinchOilPressureSwitch: %d\n", WinchOilPressureSwitch);
		printf("HydraulicOilLevel: %f %%\n",
			j1939Slot44_toDisp(HydraulicOilLevel));
	}

	if (pgn == 65212u)  {
		printf("TotalCompressionBrakeDistance %f\n",
			j1939Slot36_toDisp(TotalCompressionBrakeDistance));
		printf("TripCompressionBrakeDistance %f\n",
			j1939Slot36_toDisp(TripCompressionBrakeDistance));
		printf("TripSeviceBrakeApplication %f\n",
			j1939Slot36_toDisp(TripSeviceBrakeApplication));
		printf("TripSeviceBrakeDistance %d\n", TripSeviceBrakeDistance);
	}
}


/*********************************************************************/
/*
 * registered function for transmit PGNs
 * (Called before PGN is transmitted)
 */
static RET_T transmitPgn(
		UNSIGNED32  pgn
	)
{
static UNSIGNED8		percent = 50;
static UNSIGNED8		temperature = 10;

	printf("PGN %d to transmit\n", pgn);

	if (pgn == 65262u)  {
		temperature++;
		if (temperature > 100)  {
			temperature = 10;
		}
		ect = j1939Slot67_toCAN(temperature); /* Engine Coolant Temperature */
		ft = j1939Slot67_toCAN(temperature);  /* Fuel Temperature */
		eol = j1939Slot68_toCAN(temperature); /* Engine Oil Temperature 1 */
		tol = j1939Slot68_toCAN(temperature); /* Turbo Oil Temperature */
		eit = j1939Slot67_toCAN(temperature); /* Engine Intercooler Temperature */
		eito = j1939Slot44_toCAN(temperature);/* Engine Intercooler Thermostat Opening */
	}

	if (pgn == 53248u)  {
		percent++;
		if (percent == 100)  {
			percent = 0;
		}
		ilbp = j1939Slot44_toCAN(percent);	/* Illumination Brightness */
		sbbp = j1939Slot44_toCAN(percent);	/* switch backlight Brightness */
		sibp = j1939Slot44_toCAN(percent);	/* switch indication Brightness */
//j1939SendCmdt(64956, 27);
//j1939SendBam(64956);
	}

	return(RET_OK);
}


/*********************************************************************/
/*
 * registered function for CAN events
 */
static void canInd(
		CO_CAN_STATE_T  canState
	)
{
	switch (canState)  {
	case CO_CAN_STATE_BUS_OFF:
		printf("CAN: Bus Off\n");
		break;
	case CO_CAN_STATE_BUS_ON:
		printf("CAN: Bus On\n");
		break;
	case CO_CAN_STATE_PASSIVE:
		printf("CAN: Passive\n");
		break;
	default:
		break;
	}
}


/*********************************************************************/
/*
 * registered function for Communication events
 */
static void commInd(
		CO_COMM_STATE_EVENT_T   commEvent
	)
{
	switch (commEvent)  {
	case CO_COMM_STATE_EVENT_CAN_OVERRUN:
		printf("COMM-Event: CAN Overrun\n");
		break;
	case CO_COMM_STATE_EVENT_REC_QUEUE_FULL:
		printf("COMM-Event: Rec Queue Full\n");
		break;
	case CO_COMM_STATE_EVENT_REC_QUEUE_OVERFLOW:
		printf("COMM-Event: Rec Queue Overflow\n");
		break;
	case CO_COMM_STATE_EVENT_REC_QUEUE_EMPTY:
		printf("COMM-Event: Rec Queue Empty\n");
		break;
	case CO_COMM_STATE_EVENT_TR_QUEUE_FULL:
		printf("COMM-Event: Tr Queue Full\n");
		break;
	case CO_COMM_STATE_EVENT_TR_QUEUE_OVERFLOW:
		printf("COMM-Event: Tr Queue Overflow\n");
		break;
	case CO_COMM_STATE_EVENT_TR_QUEUE_EMPTY:
		printf("COMM-Event: Tr Queue Empty\n");
		break;
	case CO_COMM_STATE_EVENT_DRV_ERROR:
		printf("COMM-Event: Internal Driver Error\n");
		break;
	default:
		printf("COMM-Event: %d\n", commEvent);
		break;
	}
}


/*********************************************************************/
static void errorHandler(
		UNSIGNED8 error
	)
{
	printf("exit with error: %u\n", error);

	while (1)  {
		/* do nothing */
	}
}
